# modules

__all__ = ["classification",
           "data_selection",
           "feature_analysis",
           "feature_extraction",
           "feature_selection",
           "feature_transformation",
           "img_processing",
           "io",
           "misc",
           "performance_eval"]

__version__ = '0.2.9'
